@@ -22,14 +22,14 @@
 ########################################################################
 
 use strict;
-use Getopt::Long qw(:config pass_through);
+use Getopt::Long qw(:config pass_through no_auto_abbrev);
 use IPC::Open2;
 
 my $app_name     = 'colordiff';
-my $version      = '1.0.11';
+my $version      = '1.0.13';
 my $author       = 'Dave Ewart';
 my $author_email = 'davee@sungate.co.uk';
-my $app_www      = 'http://colordiff.sourceforge.net/';
+my $app_www      = 'http://www.colordiff.org/';
 my $copyright    = '(C)2002-2012';
 my $show_banner  = 1;
 my $color_patch  = 0;
@@ -69,6 +69,7 @@ my ($setting, $value);
 my @config_files = ("$etcdir/colordiffrc");
 push (@config_files, "$ENV{HOME}/.colordiffrc") if (defined $ENV{HOME});
 my $config_file;
+my $diff_type = 'unknown';
 
 # Convert tabs to spaces
 sub expand_tabs_to_spaces ($) {
@@ -98,10 +99,84 @@ sub check_for_file_arguments {
         if ($arg !~ /^-/) {
             $nonopts++;
         }
+        if ($arg eq "--help" || $arg eq "--version" || $arg eq "-v") {
+            $nonopts++;
+        }
     }
     return $nonopts;
 }
 
+sub detect_diff_type {
+    # Two parameters:
+    #    $isref       is reference to @inputstream
+    #    $allow_diffy is flag indicating whether diffy is a
+    #                   permitted diff type
+    my $isref = shift;
+    my $allow_diffy = shift;
+    my @is = @$isref;
+
+    # This may not be perfect - should identify most reasonably
+    # formatted diffs and patches
+
+    foreach my $record (@is) {
+        # Unified diffs are the only flavour having '+++ ' or '--- '
+        # at the start of a line
+        if ($record =~ /^(\+\+\+ |--- |@@ )/) {
+            return 'diffu';
+        }
+        # Context diffs are the only flavour having '***'
+        # at the start of a line
+        elsif ($record =~ /^\*\*\*/) {
+            return 'diffc';
+        }
+        # Plain diffs have NcN, NdN and NaN etc.
+        elsif ($record =~ /^[0-9,]+[acd][0-9,]+$/) {
+            return 'diff';
+        }
+        # FIXME - This is not very specific, since the regex matches could
+        # easily match non-diff output.
+        # However, given that we have not yet matched any of the *other* diff
+        # types, this might be good enough
+        #
+        # Only pick diffy if our flag parameter indicates so
+        elsif ( ($allow_diffy == 1) && ($record =~ /(\s\|\s|\s<$|\s>\s)/) ) {
+            return 'diffy';
+        }
+        # wdiff deleted/added patterns
+        # should almost always be pairwise?
+        elsif ($record =~ /\[-.*?-\]/s
+                || $record =~ /\{\+.*?\+\}/s) {
+            return 'wdiff';
+        }
+        # FIXME - This is a bit risky, but if we haven't matched any other
+        # diff type by this stage, this line usually indicates we have
+        # debdiff output
+        elsif ($record =~ /^Control files: lines which differ/) {
+            return 'debdiff';
+        }
+    }
+    $diff_type = 'unknown';
+}
+
+my $enable_verifymode;
+my $specified_difftype;
+GetOptions(
+    # --enable-verifymode option is for testing behaviour of colordiff
+    # against standard test diffs
+    "verifymode" => \$enable_verifymode,
+    "difftype=s" => \$specified_difftype
+    # TODO - check that specified type is valid, issue warning if not
+);
+
+if (defined $enable_verifymode) {
+    # When in verify mode, to ensure consistent output we don't source
+    # any external config files, so we use built-in defaults
+    # and we add colour to our patches
+    @config_files = ();
+    $color_patch  = 1;
+    $show_banner  = 0;
+}
+
 foreach $config_file (@config_files) {
     if (open (COLORDIFFRC, "<$config_file")) {
         while (<COLORDIFFRC>) {
@@ -190,11 +265,6 @@ if ((-f STDOUT) && ($color_patch == 0)) {
     $colour{off} = '';
 }
 
-my $specified_difftype;
-GetOptions(
-    "difftype=s" => \$specified_difftype
-);
-# TODO - check that specified type is valid, issue warning if not
 
 # ----------------------------------------------------------------------------
 
@@ -229,58 +299,16 @@ else {
 
 # Input stream has been read - need to examine it
 # to determine type of diff we have.
-#
-# This may not be perfect - should identify most reasonably
-# formatted diffs and patches
 
-my $diff_type = 'unknown';
 my $record;
 my $longest_record = 0;
 
-DIFF_TYPE: foreach $record (@inputstream) {
-    if (defined $specified_difftype) {
-        $diff_type = $specified_difftype;
-        last DIFF_TYPE;
-    }
-    # Unified diffs are the only flavour having '+++ ' or '--- '
-    # at the start of a line
-    if ($record =~ /^(\+\+\+ |--- |@@ )/) {
-        $diff_type = 'diffu';
-        last DIFF_TYPE;
-    }
-    # Context diffs are the only flavour having '***'
-    # at the start of a line
-    elsif ($record =~ /^\*\*\*/) {
-        $diff_type = 'diffc';
-        last DIFF_TYPE;
-    }
-    # Plain diffs have NcN, NdN and NaN etc.
-    elsif ($record =~ /^[0-9,]+[acd][0-9,]+$/) {
-        $diff_type = 'diff';
-        last DIFF_TYPE;
-    }
-    # FIXME - This is not very specific, since the regex matches could
-    # easily match non-diff output.
-    # However, given that we have not yet matched any of the *other* diff
-    # types, this might be good enough
-    elsif ($record =~ /(\s\|\s|\s<$|\s>\s)/) {
-        $diff_type = 'diffy';
-        last DIFF_TYPE;
-    }
-    # wdiff deleted/added patterns
-    # should almost always be pairwise?
-    elsif ($record =~ /\[-.*?-\]/s
-           || $record =~ /\{\+.*?\+\}/s) {
-        $diff_type = 'wdiff';
-        last DIFF_TYPE;
-    }
-    # FIXME - This is a bit risky, but if we haven't matched any other
-    # diff type by this stage, this line usually indicates we have
-    # debdiff output
-    elsif ($record =~ /^Control files: lines which differ/) {
-        $diff_type = 'debdiff';
-        last DIFF_TYPE;
-    }
+if (defined $specified_difftype) {
+    $diff_type = $specified_difftype;
+}
+else {
+    # Detect diff type, diffy is permitted
+    $diff_type = detect_diff_type(\@inputstream, 1);
 }
 
 my $inside_file_old = 1;
@@ -332,6 +360,14 @@ if ($diff_type eq 'diffy') {
             }
         }
     }
+    # If we don't find a suitable separator column then
+    # we've probably misidentified the input as diffy
+    # Search stream again, this time excluding diffy
+    # as a possible outcome
+    if ($diffy_sep_col == 0) {
+        # Detect diff type, diffy is NOT permitted
+        $diff_type = detect_diff_type(\@inputstream, 0);
+    }
 }
 # ------------------------------------------------------------------------------
 
